@@ -22,6 +22,7 @@ from pandaserver.jobdispatcher import DispatcherUtils
 from pandaserver.taskbuffer import EventServiceUtils
 from pandaserver.brokerage.SiteMapper import SiteMapper
 from pandaserver.proxycache import panda_proxy_cache
+from pandaserver.srvcore import CoreUtils
 
 # logger
 _logger = PandaLogger().getLogger('JobDispatcher')
@@ -142,10 +143,10 @@ class JobDipatcher:
             self.allowedNodes = self.taskBuffer.getAllowedNodes()
         # special dipatcher parameters
         if self.specialDispatchParams is None:
-            self.specialDispatchParams = CachedObject(60*30, self.taskBuffer.getSpecialDispatchParams)
+            self.specialDispatchParams = CachedObject(60*10, self.taskBuffer.getSpecialDispatchParams)
         # site mapper cache
         if self.siteMapperCache is None:
-            self.siteMapperCache = CachedObject(60*30, self.getSiteMapper)
+            self.siteMapperCache = CachedObject(60*10, self.getSiteMapper)
         # release
         self.lock.release()
 
@@ -156,9 +157,7 @@ class JobDipatcher:
             if realDN is None:
                 realDN = response.data['prodUserID']
             # remove redundant extensions
-            realDN = re.sub('/CN=limited proxy','',realDN)
-            realDN = re.sub('(/CN=proxy)+','',realDN)
-            realDN = re.sub('(/CN=\d+)+$','',realDN)
+            realDN = CoreUtils.get_bare_dn(realDN, keep_digits=False)
             pIF = panda_proxy_cache.MyProxyInterface()
             tmpOut = pIF.retrieve(realDN,role=role)
             # not found
@@ -180,7 +179,7 @@ class JobDipatcher:
     def getJob(self, siteName, prodSourceLabel, cpu, mem, diskSpace, node, timeout, computingElement,
                atlasRelease, prodUserID, getProxyKey, countryGroup, workingGroup, allowOtherCountry,
                realDN, taskID, nJobs, acceptJson, background, resourceType, harvester_id, worker_id,
-               schedulerID, jobType):
+               schedulerID, jobType, tmpLog):
 
         t_getJob_start = time.time()
         jobs = []
@@ -194,6 +193,13 @@ class JobDipatcher:
 
         self.siteMapperCache.update()
         is_gu = self.siteMapperCache.cachedObj.getSite(siteName).is_grandly_unified()
+        in_test = self.siteMapperCache.cachedObj.getSite(siteName).status == 'test'
+
+        # change label
+        if in_test and prodSourceLabel in ['user', 'managed', 'unified']:
+            new_label = 'test'
+            tmpLog.debug("prodSourceLabel changed {} -> {}".format(prodSourceLabel, new_label))
+            prodSourceLabel = new_label
 
         # wrapper function for timeout
         tmpWrapper = _TimedMethod(self.taskBuffer.getJobs, timeout)
@@ -204,7 +210,9 @@ class JobDipatcher:
         if isinstance(tmpWrapper.result, list):
             jobs = jobs + tmpWrapper.result
         # make response
+        secrets_map = {}
         if len(jobs) > 0:
+            secrets_map = jobs.pop()
             proxyKey = jobs[-1]
             nSent = jobs[-2]
             jobs = jobs[:-2]
@@ -216,10 +224,9 @@ class JobDipatcher:
                 try:
                     response=Protocol.Response(Protocol.SC_Success)
                     response.appendJob(tmpJob, self.siteMapperCache)
-                except Exception:
-                    errtype, errvalue = sys.exc_info()[:2]
-                    tmpMsg = "getJob failed with {0} {1}".format(errtype.__name__, errvalue)
-                    _logger.error(tmpMsg + '\n' + traceback.format_exc())
+                except Exception as e:
+                    tmpMsg = "failed to get jobs with {0}".format(str(e))
+                    tmpLog.error(tmpMsg + '\n' + traceback.format_exc())
                     raise
 
                 # append nSent
@@ -227,6 +234,11 @@ class JobDipatcher:
                 # set proxy key
                 if getProxyKey:
                     response.setProxyKey(proxyKey)
+                # set secrets
+                if tmpJob.prodUserName in secrets_map and secrets_map[tmpJob.prodUserName]:
+                    response.appendNode('secrets', secrets_map[tmpJob.prodUserName])
+                if panda_config.pilot_secrets in secrets_map and secrets_map[panda_config.pilot_secrets]:
+                    response.appendNode('pilotSecrets', secrets_map[panda_config.pilot_secrets])
                 # check if proxy cache is used
                 if hasattr(panda_config,'useProxyCache') and panda_config.useProxyCache is True:
                     self.specialDispatchParams.update()
@@ -248,8 +260,8 @@ class JobDipatcher:
                         else:
                             allowProxy = self.specialDispatchParams['allowProxy']
                         if compactDN not in allowProxy:
-                            _logger.warning("getJob : %s %s '%s' no permission to retrive user proxy" % (siteName,node,
-                                                                                                         compactDN))
+                            tmpLog.warning("%s %s '%s' no permission to retrieve user proxy" % (siteName, node,
+                                                                                                compactDN))
                         else:
                             if useProxyCache:
                                 tmpStat,tmpOut = self.setUserProxy(response,
@@ -258,24 +270,22 @@ class JobDipatcher:
                             else:
                                 tmpStat,tmpOut = self.setUserProxy(response)
                             if not tmpStat:
-                                _logger.warning("getJob : %s %s failed to get user proxy : %s" % (siteName,node,
-                                                                                                  tmpOut))
-                    except Exception:
-                        errtype,errvalue = sys.exc_info()[:2]
-                        _logger.warning("getJob : %s %s failed to get user proxy with %s:%s" % (siteName,node,
-                                                                                                errtype.__name__, errvalue))
+                                tmpLog.warning("%s %s failed to get user proxy : %s" % (siteName, node,
+                                                                                        tmpOut))
+                    except Exception as e:
+                        tmpLog.warning("%s %s failed to get user proxy with %s" % (siteName, node, str(e)))
                 # panda proxy
                 if 'pandaProxySites' in self.specialDispatchParams and siteName in self.specialDispatchParams['pandaProxySites'] \
                         and (EventServiceUtils.isEventServiceJob(tmpJob) or EventServiceUtils.isEventServiceMerge(tmpJob)):
                     # get secret key
                     tmpSecretKey,tmpErrMsg = DispatcherUtils.getSecretKey(tmpJob.PandaID)
                     if tmpSecretKey is None:
-                        _logger.warning("getJob : PandaID=%s site=%s failed to get panda proxy secret key : %s" % (tmpJob.PandaID,
-                                                                                                                   siteName,
-                                                                                                                   tmpErrMsg))
+                        tmpLog.warning("PandaID=%s site=%s failed to get panda proxy secret key : %s" % (tmpJob.PandaID,
+                                                                                                         siteName,
+                                                                                                         tmpErrMsg))
                     else:
                         # set secret key
-                        _logger.debug("getJob : PandaID=%s key=%s" % (tmpJob.PandaID,tmpSecretKey))
+                        tmpLog.debug("PandaID=%s set key=%s" % (tmpJob.PandaID, tmpSecretKey))
                         response.setPandaProxySecretKey(tmpSecretKey)
                 # add
                 responseList.append(response.data)
@@ -287,10 +297,9 @@ class JobDipatcher:
                         response.appendNode('jobs',json.dumps(responseList))
                     else:
                         response.appendNode('jobs',responseList)
-                except Exception:
-                    errtype, errvalue = sys.exc_info()[:2]
-                    tmpMsg = "getJob failed with {0} {1}".format(errtype.__name__, errvalue)
-                    _logger.error(tmpMsg + '\n' + traceback.format_exc())
+                except Exception as e:
+                    tmpMsg = "failed to make response with {0}".format(str(e))
+                    tmpLog.error(tmpMsg + '\n' + traceback.format_exc())
                     raise
 
         else:
@@ -308,11 +317,12 @@ class JobDipatcher:
                     response = Protocol.Response(Protocol.SC_NoJobs)
                 _pilotReqLogger.info('method=noJob,site=%s,node=%s,type=%s' % (siteName, node, prodSourceLabel))
         # return
-        _logger.debug("getJob : %s %s ret -> %s" % (siteName, node, response.encode(acceptJson)))
+        tmpLog.debug("%s %s ret -> %s" % (siteName, node, response.encode(acceptJson)))
 
         t_getJob_end = time.time()
         t_getJob_spent = t_getJob_end - t_getJob_start
-        _logger.info("getJob : siteName={0} took timing={1}s".format(siteName, t_getJob_spent))
+        tmpLog.info("siteName={0} took timing={1}s in_test={2}".format(siteName, t_getJob_spent,
+                                                                       in_test))
         return response.encode(acceptJson)
 
 
@@ -362,13 +372,21 @@ class JobDipatcher:
             if tmpWrapper.result:
                 # succeed
                 response=Protocol.Response(Protocol.SC_Success)
+                result = tmpWrapper.result
+                secrets = None
+                if isinstance(result, dict):
+                    if 'secrets' in result:
+                        secrets = result['secrets']
+                    result = result['command']
                 # set command
-                if isinstance(tmpWrapper.result, str):
-                    response.appendNode('command',tmpWrapper.result)
+                if isinstance(result, str):
+                    response.appendNode('command', result)
+                    if secrets:
+                        response.appendNode('pilotSecrets', secrets)
                 else:
-                    response.appendNode('command','NULL')
+                    response.appendNode('command', 'NULL')
                 # add output to dataset
-                if tmpWrapper.result not in ["badattemptnr","alreadydone"] and (jobStatus == 'failed' or jobStatus == 'finished'):
+                if result not in ["badattemptnr", "alreadydone"] and (jobStatus == 'failed' or jobStatus == 'finished'):
                     adder_gen = AdderGen(self.taskBuffer, jobID, jobStatus, attemptNr)
                     adder_gen.dumpFileReport(xml, attemptNr)
                     del adder_gen
@@ -680,12 +698,13 @@ class JobDipatcher:
     def getProxy(self, realDN, role, targetDN):
         if targetDN is None:
             targetDN = realDN
-        tmpMsg = "getProxy DN={0} role={1} target={2} ".format(realDN, role, targetDN)
-        _logger.debug(tmpMsg)
+        tmpLog = LogWrapper(_logger, 'getProxy PID={}'.format(os.getpid()))
+        tmpMsg = 'start DN="{}" role={} target="{}" '.format(realDN, role, targetDN)
+        tmpLog.debug(tmpMsg)
         if realDN is None:
             # cannot extract DN
             tmpMsg += "failed since DN cannot be extracted"
-            _logger.debug(tmpMsg)
+            tmpLog.debug(tmpMsg)
             response = Protocol.Response(Protocol.SC_Perms,'Cannot extract DN from proxy. not HTTPS?')
         else:
             # get compact DN
@@ -701,18 +720,18 @@ class JobDipatcher:
                 tmpMsg += "failed since '{0}' not in the authorized user list who have 'p' in {1}.USERS.GRIDPREF ".format(compactDN,
                                                                                                                           panda_config.schemaMETA)
                 tmpMsg += "to get proxy"
-                _logger.debug(tmpMsg)
+                tmpLog.debug(tmpMsg)
                 response = Protocol.Response(Protocol.SC_Perms,tmpMsg)
             else:
                 # get proxy
                 response = Protocol.Response(Protocol.SC_Success,'')
                 tmpStat,tmpMsg = self.setUserProxy(response, targetDN, role)
                 if not tmpStat:
-                    _logger.debug(tmpMsg)
+                    tmpLog.debug(tmpMsg)
                     response.appendNode('StatusCode',Protocol.SC_ProxyError)
                 else:
-                    tmpMsg = "sent proxy"
-                    _logger.debug(tmpMsg)
+                    tmpMsg = "successful sent proxy"
+                    tmpLog.debug(tmpMsg)
         # return
         return response.encode(True)
 
@@ -720,6 +739,24 @@ class JobDipatcher:
     def getActiveJobAttributes(self, pandaID, attrs):
         return self.taskBuffer.getActiveJobAttributes(pandaID, attrs)
 
+    # update job status
+    def updateWorkerPilotStatus(self, workerID, harvesterID, status, timeout, accept_json):
+
+        tmp_wrapper = _TimedMethod(self.taskBuffer.updateWorkerPilotStatus, timeout)
+        tmp_wrapper.run(workerID, harvesterID, status)
+
+        # make response
+        if tmp_wrapper.result == Protocol.TimeOutToken:
+            response = Protocol.Response(Protocol.SC_TimeOut)
+        else:
+            if tmp_wrapper.result:  # success
+                response = Protocol.Response(Protocol.SC_Success)
+            else:  # error
+                response = Protocol.Response(Protocol.SC_Failed)
+        _logger.debug("updateWorkerPilotStatus : {0} {1} {2} ret -> {3}"
+                      .format(workerID, harvesterID, status, response.encode(accept_json)))
+        return response.encode(accept_json)
+
 
 # Singleton
 jobDispatcher = JobDipatcher()
@@ -759,7 +796,7 @@ def _checkRole(fqans,dn,jdCore,withVomsPatch=True,site='',hostname=''):
                      '/atlas/Role=production',
                      '/atlas/Role=pilot',
                      '/osg/Role=pilot',
-                     '/cms/Role=pilot',
+                     '^/[^/]+/Role=production$',
                      '/ams/Role=pilot',
                      '/Engage/LBNE/Role=pilot',
                      ]
@@ -774,6 +811,9 @@ def _checkRole(fqans,dn,jdCore,withVomsPatch=True,site='',hostname=''):
                 if fqan.startswith(rolePat):
                     prodManager = True
                     break
+                if re.search(rolePat, fqan):
+                    prodManager = True
+                    break
             # escape
             if prodManager:
                 break
@@ -814,8 +854,7 @@ def _getDN(req):
     if 'SSL_CLIENT_S_DN' in req.subprocess_env:
         realDN = req.subprocess_env['SSL_CLIENT_S_DN']
         # remove redundant CN
-        realDN = re.sub('/CN=limited proxy','',realDN)
-        realDN = re.sub('/CN=proxy(/CN=proxy)+','/CN=proxy',realDN)
+        realDN = CoreUtils.get_bare_dn(realDN, keep_proxy=True)
     # return
     return realDN
 
@@ -841,7 +880,8 @@ def getJob(req, siteName, token=None, timeout=60, cpu=None, mem=None, diskSpace=
            computingElement=None, AtlasRelease=None, prodUserID=None, getProxyKey=None, countryGroup=None,
            workingGroup=None, allowOtherCountry=None, taskID=None, nJobs=None, background=None, resourceType=None,
            harvester_id=None, worker_id=None, schedulerID=None, jobType=None):
-    _logger.debug("getJob(%s)" % siteName)
+    tmpLog = LogWrapper(_logger, 'getJob {}'.format(datetime.datetime.utcnow().isoformat('/')))
+    tmpLog.debug(siteName)
     # get DN
     realDN = _getDN(req)
     # get FQANs
@@ -880,12 +920,13 @@ def getJob(req, siteName, token=None, timeout=60, cpu=None, mem=None, diskSpace=
         background = True
     else:
         background = False
-    _logger.debug("getJob(%s,nJobs=%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,taskID=%s,DN:%s,role:%s,token:%s,val:%s,FQAN:%s,json:%s,bg=%s,rt=%s," \
-                      "harvester_id=%s,worker_id=%s,schedulerID=%s,jobType=%s" \
-                  % (siteName, nJobs, cpu, mem, diskSpace, prodSourceLabel, node,
-                     computingElement, AtlasRelease, prodUserID, getProxyKey, countryGroup, workingGroup,
-                     allowOtherCountry, taskID, realDN, prodManager, token, validToken, str(fqans), req.acceptJson(),
-                     background, resourceType, harvester_id, worker_id, schedulerID, jobType))
+    tmpLog.debug("%s,nJobs=%s,cpu=%s,mem=%s,disk=%s,source_label=%s,node=%s,ce=%s,rel=%s,user=%s,proxy=%s,c_group=%s,"
+                 "w_group=%s,%s,taskID=%s,DN:%s,role:%s,token:%s,val:%s,FQAN:%s,json:%s,bg=%s,rt=%s,"
+                 "harvester_id=%s,worker_id=%s,schedulerID=%s,jobType=%s"
+                 % (siteName, nJobs, cpu, mem, diskSpace, prodSourceLabel, node,
+                    computingElement, AtlasRelease, prodUserID, getProxyKey, countryGroup, workingGroup,
+                    allowOtherCountry, taskID, realDN, prodManager, token, validToken, str(fqans), req.acceptJson(),
+                    background, resourceType, harvester_id, worker_id, schedulerID, jobType))
     try:
         dummyNumSlots = int(nJobs)
     except Exception:
@@ -897,7 +938,7 @@ def getJob(req, siteName, token=None, timeout=60, cpu=None, mem=None, diskSpace=
         _pilotReqLogger.info('method=getJob,site=%s,node=%s,type=%s' % (siteName,node,prodSourceLabel))
     # invalid role
     if (not prodManager) and (prodSourceLabel not in ['user']):
-        _logger.warning("getJob(%s) : invalid role" % siteName)
+        tmpLog.warning("invalid role")
         if req.acceptJson():
             tmpMsg = 'no production/pilot role in VOMS FQANs or non pilot owner'
         else:
@@ -905,13 +946,14 @@ def getJob(req, siteName, token=None, timeout=60, cpu=None, mem=None, diskSpace=
         return Protocol.Response(Protocol.SC_Role, tmpMsg).encode(req.acceptJson())
     # invalid token
     if not validToken:
-        _logger.warning("getJob(%s) : invalid token" % siteName)
+        tmpLog.warning("invalid token")
         return Protocol.Response(Protocol.SC_Invalid).encode(req.acceptJson())
     # invoke JD
     return jobDispatcher.getJob(siteName, prodSourceLabel, cpu, mem, diskSpace, node, int(timeout),
                                 computingElement, AtlasRelease, prodUserID, getProxyKey, countryGroup,
                                 workingGroup, allowOtherCountry, realDN, taskID, nJobs, req.acceptJson(),
-                                background, resourceType, harvester_id, worker_id, schedulerID, jobType)
+                                background, resourceType, harvester_id, worker_id, schedulerID, jobType,
+                                tmpLog)
 
 
 # update job status
@@ -924,7 +966,7 @@ def updateJob(req, jobId, state, token=None, transExitCode=None, pilotErrorCode=
               coreCount=None, maxRSS=None, maxVMEM=None, maxSWAP=None, maxPSS=None, avgRSS=None, avgVMEM=None,
               avgSWAP=None, avgPSS=None, totRCHAR=None, totWCHAR=None, totRBYTES=None, totWBYTES=None, rateRCHAR=None,
               rateWCHAR=None, rateRBYTES=None, rateWBYTES=None, corruptedFiles=None, meanCoreCount=None):
-    tmpLog = LogWrapper(_logger,'updateJob PandaID={0} PID={1}'.format(jobId,os.getpid()))
+    tmpLog = LogWrapper(_logger, 'updateJob PandaID={0} PID={1}'.format(jobId,os.getpid()))
     tmpLog.debug('start')
     # get DN
     realDN = _getDN(req)
@@ -1157,7 +1199,6 @@ def getEventRanges(req, pandaID, jobsetID, taskID=None, nRanges=10, timeout=60,
                                         scattered, segment_id)
 
 
-
 # update an event range
 def updateEventRange(req, eventRangeID, eventStatus, coreCount=None, cpuConsumptionTime=None,
                      objstoreID=None, timeout=60, pandaID=None):
@@ -1178,7 +1219,6 @@ def updateEventRange(req, eventRangeID, eventStatus, coreCount=None, cpuConsumpt
                                           objstoreID, int(timeout))
 
 
-
 # update an event ranges
 def updateEventRanges(req, eventRanges, timeout=120, version=0, pandaID=None):
     tmpStr = "updateEventRanges(%s)" % eventRanges
@@ -1200,7 +1240,6 @@ def updateEventRanges(req, eventRanges, timeout=120, version=0, pandaID=None):
     return jobDispatcher.updateEventRanges(eventRanges,int(timeout),req.acceptJson(),version)
 
 
-
 # check event availability
 def checkEventsAvailability(req, pandaID, jobsetID, taskID, timeout=60):
     tmpStr = "checkEventsAvailability(pandaID={0} jobsetID={1} taskID={2})".format(pandaID, jobsetID, taskID)
@@ -1212,7 +1251,6 @@ def checkEventsAvailability(req, pandaID, jobsetID, taskID, timeout=60):
     return jobDispatcher.checkEventsAvailability(pandaID, jobsetID, taskID, timeout)
 
 
-
 # generate pilot token
 def genPilotToken(req, schedulerid, host=None):
     # get DN
@@ -1232,7 +1270,6 @@ def genPilotToken(req, schedulerid, host=None):
     return jobDispatcher.genPilotToken(host,realDN,schedulerid)
 
 
-
 # get key pair
 def getKeyPair(req, publicKeyName, privateKeyName):
     # get DN
@@ -1240,7 +1277,6 @@ def getKeyPair(req, publicKeyName, privateKeyName):
     return jobDispatcher.getKeyPair(realDN,publicKeyName,privateKeyName,req.acceptJson())
 
 
-
 # get proxy
 def getProxy(req, role=None, dn=None):
     # get DN
@@ -1317,3 +1353,30 @@ def getResourceTypes(req, timeout=30):
     accept_json = req.acceptJson()
     # retrieve the commands
     return jobDispatcher.getResourceTypes(timeout, accept_json)
+
+
+# update the status of a worker according to the pilot
+def updateWorkerPilotStatus(req, site, workerID, harvesterID, status, timeout=60):
+
+    tmp_log = LogWrapper(_logger, 'updateWorkerPilotStatus workerID={0} harvesterID={1} status={2} PID={3}'
+                         .format(workerID, harvesterID, status, os.getpid()))
+    tmp_log.debug('start')
+
+    # validate the pilot permissions
+    tmp_stat, tmp_out = checkPilotPermission(req, site)
+    if not tmp_stat:
+        tmp_log.error('failed with {0}'.format(tmp_out))
+        return tmp_out
+
+    # validate the state passed by the pilot
+    valid_states = ('started', 'finished')
+    if status not in valid_states:
+        message = 'Invalid worker state. The worker state has to be in {0}'.format(valid_worker_states)
+        tmp_log.debug(message)
+        return message
+
+    accept_json = req.acceptJson()
+
+    return jobDispatcher.updateWorkerPilotStatus(workerID, harvesterID, status, timeout, accept_json)
+
+
