@@ -16,6 +16,7 @@ import gzip
 
 # config file
 from pandaserver.config import panda_config
+from pandaserver.srvcore import CoreUtils
 
 from pandaserver.taskbuffer.Initializer import initializer
 from pandaserver.taskbuffer.TaskBuffer import taskBuffer
@@ -28,7 +29,7 @@ from pandaserver.taskbuffer.Utils import isAlive, putFile, deleteFile, getServer
 from pandaserver.dataservice.DataService import datasetCompleted, updateFileStatusInDisp
 from pandaserver.jobdispatcher.JobDispatcher import getJob, updateJob, getStatus, genPilotToken,\
     getEventRanges, updateEventRange, getKeyPair, updateEventRanges, getDNsForS3, getProxy, getCommands, ackCommands,\
-    checkJobStatus, checkEventsAvailability, updateJobsInBulk, getResourceTypes
+    checkJobStatus, checkEventsAvailability, updateJobsInBulk, getResourceTypes, updateWorkerPilotStatus
 from pandaserver.userinterface.UserIF import submitJobs, getJobStatus, queryPandaIDs, killJobs, reassignJobs,\
      getJobStatistics, getJobStatisticsPerSite, resubmitJobs, queryLastFilesInDataset, getPandaIDsSite,\
      getJobsToBeUpdated, updateProdDBUpdateTimes, runTaskAssignment, getAssigningTask, getSiteSpecs,\
@@ -46,10 +47,13 @@ from pandaserver.userinterface.UserIF import submitJobs, getJobStatus, queryPand
      pauseTask, resumeTask, increaseAttemptNrPanda, killUnfinishedJobs, changeTaskSplitRulePanda,\
      changeTaskModTimePanda, avalancheTask, getPandaIDsWithTaskID, reactivateTask, getTaskStatus, \
      reassignShare, listTasksInShare, getTaskParamsMap, updateWorkers, harvesterIsAlive,\
-     reportWorkerStats, reportWorkerStats_jobtype, addHarvesterDialogs, getJobStatisticsPerSiteResource, setNumSlotsForWP,\
-     reloadInput, enableJumboJobs, updateServiceMetrics, getUserJobMetadata, getJumboJobDatasets, getGShareStatus,\
+     reportWorkerStats, reportWorkerStats_jobtype, addHarvesterDialogs, \
+     getJobStatisticsPerSiteResource, setNumSlotsForWP, reloadInput, enableJumboJobs, updateServiceMetrics, \
+     getUserJobMetadata, getJumboJobDatasets, getGShareStatus,\
      sweepPQ,get_job_statistics_per_site_label_resource, relay_idds_command, send_command_to_job,\
-     execute_idds_workflow_command
+     execute_idds_workflow_command, set_user_secret, get_user_secrets, get_ban_users
+
+from pandaserver.userinterface import Client
 
 # import error
 import pandaserver.taskbuffer.ErrorCode
@@ -86,7 +90,8 @@ allowedMethods += ['datasetCompleted', 'updateFileStatusInDisp']
 allowedMethods += ['getJob', 'updateJob', 'getStatus', 'genPilotToken',
                    'getEventRanges', 'updateEventRange', 'getKeyPair',
                    'updateEventRanges', 'getDNsForS3', 'getProxy', 'getCommands', 'ackCommands',
-                   'checkJobStatus', 'checkEventsAvailability', 'updateJobsInBulk', 'getResourceTypes']
+                   'checkJobStatus', 'checkEventsAvailability', 'updateJobsInBulk', 'getResourceTypes',
+                   'updateWorkerPilotStatus']
 
 allowedMethods += ['submitJobs', 'getJobStatus', 'queryPandaIDs', 'killJobs', 'reassignJobs', 'getJobStatistics',
                    'getJobStatisticsPerSite', 'resubmitJobs', 'queryLastFilesInDataset', 'getPandaIDsSite',
@@ -110,7 +115,8 @@ allowedMethods += ['submitJobs', 'getJobStatus', 'queryPandaIDs', 'killJobs', 'r
                    'getJobStatisticsPerSiteResource', 'setNumSlotsForWP', 'reloadInput', 'enableJumboJobs',
                    'updateServiceMetrics', 'getUserJobMetadata', 'getJumboJobDatasets',
                    'getGShareStatus', 'sweepPQ', 'get_job_statistics_per_site_label_resource', 'relay_idds_command',
-                   'send_command_to_job','execute_idds_workflow_command']
+                   'send_command_to_job','execute_idds_workflow_command', 'set_user_secret', 'get_user_secrets',
+                   'get_ban_users']
 
 
 # FastCGI/WSGI entry
@@ -131,6 +137,13 @@ if panda_config.useFastCGI or panda_config.useWSGI:
     # logger
     _logger = PandaLogger().getLogger('Entry')
 
+    # ban list
+    if panda_config.nDBConnection != 0:
+        # get ban list directly from the database
+        ban_user_list = CoreUtils.CachedObject('ban_list', 600, taskBuffer.get_ban_users, _logger)
+    else:
+        # get ban list from remote
+        ban_user_list = CoreUtils.CachedObject('ban_list', 600, Client.get_ban_users, _logger)
 
     # dummy request object
     class DummyReq:
@@ -160,20 +173,39 @@ if panda_config.useFastCGI or panda_config.useWSGI:
                         token = oidc_utils.deserialize_token(serialized_token, panda_config.auth_config,
                                                              vo)
                     # check with auth policies
+                    role = None
                     if panda_config.token_authType == 'oidc':
                         self.authenticated = False
-                        vo = token[ "vo"]
+                        # extract role
+                        if vo:
+                            vo = token["vo"].split(':')[0]
+                            vo = vo.split('.')[0]
+                            if vo != token["vo"]:
+                                role = token["vo"].split(':')[-1]
+                                role = role.split('.')[-1]
+                        # check vo
                         if vo not in panda_config.auth_policies:
                             self.message = 'unknown vo : {}'.format(vo)
                             tmpLog.error('{} - {}'.format(self.message, env['HTTP_AUTHORIZATION']))
                         else:
-                            for memberStr, memberInfo in panda_config.auth_policies[vo]:
-                                if memberStr in token["groups"]:
+                            # check role
+                            if role:
+                                if '{}/{}'.format(vo, role) not in token["groups"]:
+                                    self.message = 'not in the {}/{} group'.format(vo, role)
+                                    tmpLog.error('{} - {}'.format(self.message, env['HTTP_AUTHORIZATION']))
+                                else:
                                     self.subprocess_env['PANDA_OIDC_VO'] = vo
-                                    self.subprocess_env['PANDA_OIDC_GROUP'] = memberInfo['group']
-                                    self.subprocess_env['PANDA_OIDC_ROLE'] = memberInfo['role']
+                                    self.subprocess_env['PANDA_OIDC_GROUP'] = role
+                                    self.subprocess_env['PANDA_OIDC_ROLE'] = role
                                     self.authenticated = True
-                                    break
+                            else:
+                                for memberStr, memberInfo in panda_config.auth_policies[vo]:
+                                    if memberStr in token["groups"]:
+                                        self.subprocess_env['PANDA_OIDC_VO'] = vo
+                                        self.subprocess_env['PANDA_OIDC_GROUP'] = memberInfo['group']
+                                        self.subprocess_env['PANDA_OIDC_ROLE'] = memberInfo['role']
+                                        self.authenticated = True
+                                        break
                             if not self.authenticated:
                                 self.message = 'invalid member in {}'.format(vo)
                                 tmpLog.error('{} - {}'.format(self.message, env['HTTP_AUTHORIZATION']))
@@ -193,13 +225,21 @@ if panda_config.useFastCGI or panda_config.useWSGI:
                             if 'name' in token:
                                 self.subprocess_env['SSL_CLIENT_S_DN'] = ' '.join(
                                     [t[:1].upper() + t[1:].lower() for t in str(token['name']).split()])
+                                if 'preferred_username' in token:
+                                    self.subprocess_env['SSL_CLIENT_S_DN'] += \
+                                        '/CN=nickname:{}'.format(token['preferred_username'])
                             else:
                                 self.subprocess_env['SSL_CLIENT_S_DN'] = str(token['sub'])
                             i = 0
                             for scope in token.get('scope', '').split():
                                 if scope.startswith('role:'):
-                                    self.subprocess_env['GRST_CRED_AUTH_TOKEN_{0}'.format(i)] = 'VOMS ' + str(scope.split(':')[-1])
+                                    self.subprocess_env['GRST_CRED_AUTH_TOKEN_{0}'.format(i)] = \
+                                        'VOMS ' + str(scope.split(':')[-1])
                                     i += 1
+                            if role:
+                                self.subprocess_env['GRST_CRED_AUTH_TOKEN_{0}'.format(i)] = \
+                                    'VOMS /{}/Role={}'.format(vo, role)
+                                i += 1
             except Exception as e:
                 self.message = 'invalid token: {}'.format(str(e))
                 tmpLog.error('{} - {}'.format(self.message, env['HTTP_AUTHORIZATION']))
@@ -250,6 +290,21 @@ if panda_config.useFastCGI or panda_config.useWSGI:
             else:
                 body = b''
                 try:
+                    # dummy request object
+                    dummyReq = DummyReq(environ, tmpLog)
+                    if not dummyReq.authenticated:
+                        start_response('403 Forbidden', [('Content-Type', 'text/plain')])
+                        return ["ERROR : token-based authentication failed on the server side with {}".format(
+                            dummyReq.message).encode()]
+                    username = dummyReq.subprocess_env.get('SSL_CLIENT_S_DN', None)
+                    if username:
+                        username = CoreUtils.clean_user_id(username)
+                        if username in ban_user_list:
+                            errMsg = '{} is banned'.format(username)
+                            tmpLog.warning(errMsg)
+                            start_response('403 Forbidden', [('Content-Type', 'text/plain')])
+                            return ["ERROR : {}".format(errMsg).encode()]
+                    # read contents
                     while cont_length > 0:
                         chunk = environ['wsgi.input'].read(min(cont_length, 1024*1024))
                         if not chunk:
@@ -279,12 +334,6 @@ if panda_config.useFastCGI or panda_config.useWSGI:
                         params = json.loads(body)
                     if panda_config.entryVerbose:
                         tmpLog.debug("with %s" % str(list(params)))
-                    # dummy request object
-                    dummyReq = DummyReq(environ, tmpLog)
-                    if not dummyReq.authenticated:
-                        start_response('403 Forbidden', [('Content-Type', 'text/plain')])
-                        return ["ERROR : token-based authentication failed on the server side with {}".format(
-                            dummyReq.message).encode()]
                     param_list = [dummyReq]
                     # exec
                     exeRes = tmpMethod(*param_list, **params)
