@@ -18,6 +18,7 @@ from pandaserver.brokerage.SiteMapper import SiteMapper
 from pandaserver.taskbuffer import PrioUtil, JobUtils
 from pandaserver.dataservice.DDM import rucioAPI
 from pandaserver.config import panda_config
+from pandaserver.srvcore import CoreUtils
 
 from pandacommon.pandalogger.PandaLogger import PandaLogger
 from pandacommon.pandalogger.LogWrapper import LogWrapper
@@ -480,11 +481,9 @@ class UserIF:
         try:
             if useMailAsID:
                 _logger.debug("killJob : getting mail address for %s" % user)
-                realDN = re.sub('/CN=limited proxy','',user)
-                realDN = re.sub('(/CN=proxy)+','',realDN)
                 nTry = 3
                 for iDDMTry in range(nTry):
-                    status,userInfo = rucioAPI.finger(realDN)
+                    status,userInfo = rucioAPI.finger(user)
                     if status:
                         _logger.debug("killJob : %s is converted to %s" % (user,userInfo['email']))
                         user = userInfo['email']
@@ -609,6 +608,11 @@ class UserIF:
         # return
         return ret
 
+    # get ban users
+    def get_ban_users(self):
+        ret = self.taskBuffer.get_ban_users()
+        # return
+        return json.dumps(ret)
 
     # get client version
     def getPandaClientVer(self):
@@ -1029,6 +1033,18 @@ class UserIF:
         # return
         return ret
 
+    # set user secret
+    def set_user_secret(self, owner, key, value):
+        ret = self.taskBuffer.set_user_secret(owner, key, value)
+        # return
+        return ret
+
+    # get user secrets
+    def get_user_secrets(self, owner):
+        ret = self.taskBuffer.get_user_secrets(owner)
+        # return
+        return ret
+
 
 # Singleton
 userIF = UserIF()
@@ -1062,15 +1078,13 @@ def _getFQAN(req):
 def _getDN(req):
     realDN = ''
     if 'SSL_CLIENT_S_DN' in req.subprocess_env:
-        realDN = req.subprocess_env['SSL_CLIENT_S_DN']
         # remove redundant CN
-        realDN = re.sub('/CN=limited proxy','',realDN)
-        realDN = re.sub('/CN=proxy(/CN=proxy)+','/CN=proxy',realDN)
+        realDN = CoreUtils.get_bare_dn(req.subprocess_env['SSL_CLIENT_S_DN'], keep_proxy=True)
     return realDN
 
 
 # check role
-def _isProdRoleATLAS(req):
+def _hasProdRole(req):
     # check DN
     user = _getDN(req)
     for sdn in panda_config.production_dns:
@@ -1081,9 +1095,11 @@ def _isProdRoleATLAS(req):
     # loop over all FQANs
     for fqan in fqans:
         # check production role
-        for rolePat in ['/atlas/usatlas/Role=production','/atlas/Role=production']:
+        for rolePat in ['/atlas/usatlas/Role=production', '/atlas/Role=production', '^/[^/]+/Role=production$']:
             if fqan.startswith(rolePat):
                 return True
+            if re.search(rolePat, fqan):
+                return True
     return False
 
 
@@ -1135,7 +1151,7 @@ def submitJobs(req,jobs,toPending=None):
     # hostname
     host = req.get_remote_host()
     # production Role
-    prodRole = _isProdRoleATLAS(req)
+    prodRole = _hasProdRole(req)
     # to pending
     if toPending == 'True':
         toPending = True
@@ -1196,7 +1212,7 @@ def setCloudTaskByUser(req,tid,cloud='',status=''):
         return "ERROR: SSL connection is required"
     user = _getDN(req)
     # check role
-    if not _isProdRoleATLAS(req):
+    if not _hasProdRole(req):
         return "ERROR: production role is required"
     return userIF.setCloudTaskByUser(user,tid,cloud,status)
 
@@ -1208,7 +1224,7 @@ def setDebugMode(req,pandaID,modeOn):
         return "ERROR: SSL connection is required"
     user = _getDN(req)
     # check role
-    prodManager = _isProdRoleATLAS(req)
+    prodManager = _hasProdRole(req)
     # mode
     if modeOn == 'True':
         modeOn = True
@@ -1227,7 +1243,7 @@ def insertSandboxFileInfo(req,userName,fileName,fileSize,checkSum):
         return "ERROR: SSL connection is required"
     user = _getDN(req)
     # check role
-    prodManager = _isProdRoleATLAS(req)
+    prodManager = _hasProdRole(req)
     if not prodManager:
         return "ERROR: missing role"
     # hostname
@@ -1389,7 +1405,7 @@ def killJobs(req,ids,code=None,useMailAsID=None,killOpts=None):
     if 'SSL_CLIENT_S_DN' in req.subprocess_env:
         user = _getDN(req)
     # check role
-    prodManager = _isProdRoleATLAS(req)
+    prodManager = _hasProdRole(req)
     # get FQANs
     fqans = _getFQAN(req)
     # use email address as ID
@@ -1449,7 +1465,7 @@ def changeJobPriorities(req,newPrioMap=None):
     if 'SSL_CLIENT_S_DN' in req.subprocess_env:
         user = _getDN(req)
     # check role
-    prodRole = _isProdRoleATLAS(req)
+    prodRole = _hasProdRole(req)
     ret = userIF.changeJobPriorities(user,prodRole,newPrioMap)
     return WrappedPickle.dumps(ret)
 
@@ -1473,6 +1489,10 @@ def getCachePrefixes(req):
 def getCmtConfigList(self,relaseVer):
     return userIF.getCmtConfigList(relaseVer)
 
+# get ban users
+def get_ban_users(req):
+    return userIF.get_ban_users()
+
 # get client version
 def getPandaClientVer(req):
     return userIF.getPandaClientVer()
@@ -1781,7 +1801,7 @@ def insertTaskParams(req, taskParams=None, properErrorCode=None, parent_tid=None
         tmpLog.debug(tmpMsg)
         return WrappedPickle.dumps((False, tmpMsg))
     # check role
-    prodRole = _isProdRoleATLAS(req)
+    prodRole = _hasProdRole(req)
     # get FQANs
     fqans = _getFQAN(req)
 
@@ -1812,7 +1832,7 @@ def killTask(req,jediTaskID=None,properErrorCode=None):
     if 'SSL_CLIENT_S_DN' in req.subprocess_env:
         user = _getDN(req)
     # check role
-    prodRole = _isProdRoleATLAS(req)
+    prodRole = _hasProdRole(req)
     # check jediTaskID
     try:
         jediTaskID = long(jediTaskID)
@@ -1856,7 +1876,7 @@ def retryTask(req,jediTaskID,properErrorCode=None,newParams=None,noChildRetry=No
     if 'SSL_CLIENT_S_DN' in req.subprocess_env:
         user = _getDN(req)
     # check role
-    prodRole = _isProdRoleATLAS(req)
+    prodRole = _hasProdRole(req)
     # check jediTaskID
     try:
         jediTaskID = long(jediTaskID)
@@ -1881,7 +1901,7 @@ def reassignTask(req,jediTaskID,site=None,cloud=None,nucleus=None,soft=None,mode
     if 'SSL_CLIENT_S_DN' in req.subprocess_env:
         user = _getDN(req)
     # check role
-    prodRole = _isProdRoleATLAS(req)
+    prodRole = _hasProdRole(req)
     # check jediTaskID
     try:
         jediTaskID = long(jediTaskID)
@@ -1924,7 +1944,7 @@ def finishTask(req,jediTaskID=None,properErrorCode=None,soft=None):
     if 'SSL_CLIENT_S_DN' in req.subprocess_env:
         user = _getDN(req)
     # check role
-    prodRole = _isProdRoleATLAS(req)
+    prodRole = _hasProdRole(req)
     # check jediTaskID
     try:
         jediTaskID = long(jediTaskID)
@@ -1955,7 +1975,7 @@ def reloadInput(req, jediTaskID, properErrorCode=None):
     if 'SSL_CLIENT_S_DN' in req.subprocess_env:
         user = _getDN(req)
     # check role
-    prodRole = _isProdRoleATLAS(req)
+    prodRole = _hasProdRole(req)
     # check jediTaskID
     try:
         jediTaskID = long(jediTaskID)
@@ -1997,7 +2017,7 @@ def changeTaskPriority(req,jediTaskID=None,newPriority=None):
     if 'SSL_CLIENT_S_DN' in req.subprocess_env:
         user = _getDN(req)
     # check role
-    prodRole = _isProdRoleATLAS(req)
+    prodRole = _hasProdRole(req)
     # only prod managers can use this method
     if not prodRole:
         return "Failed : production or pilot role required"
@@ -2026,7 +2046,7 @@ def increaseAttemptNrPanda(req,jediTaskID,increasedNr):
     if 'SSL_CLIENT_S_DN' in req.subprocess_env:
         user = _getDN(req)
     # check role
-    prodRole = _isProdRoleATLAS(req)
+    prodRole = _hasProdRole(req)
     # only prod managers can use this method
     ret = None
     if not prodRole:
@@ -2063,7 +2083,7 @@ def changeTaskAttributePanda(req,jediTaskID,attrName,attrValue):
     if 'SSL_CLIENT_S_DN' in req.subprocess_env:
         user = _getDN(req)
     # check role
-    prodRole = _isProdRoleATLAS(req)
+    prodRole = _hasProdRole(req)
     # only prod managers can use this method
     if not prodRole:
         return WrappedPickle.dumps((False,"production or pilot role required"))
@@ -2090,7 +2110,7 @@ def changeTaskSplitRulePanda(req,jediTaskID,attrName,attrValue):
     if 'SSL_CLIENT_S_DN' in req.subprocess_env:
         user = _getDN(req)
     # check role
-    prodRole = _isProdRoleATLAS(req)
+    prodRole = _hasProdRole(req)
     # only prod managers can use this method
     if not prodRole:
         return WrappedPickle.dumps((False,"production or pilot role required"))
@@ -2100,7 +2120,7 @@ def changeTaskSplitRulePanda(req,jediTaskID,attrName,attrValue):
     except Exception:
         return WrappedPickle.dumps((False,'jediTaskID must be an integer'))
     # check attribute
-    if attrName not in ['TW','EC','ES','MF','NG','NI','NF','NJ', 'AV', 'IL', 'LI']:
+    if attrName not in ['TW','EC','ES','MF','NG','NI','NF','NJ', 'AV', 'IL', 'LI', 'LC', 'CC']:
         return WrappedPickle.dumps((2,"disallowed to update {0}".format(attrName)))
     ret = userIF.changeTaskSplitRulePanda(jediTaskID,attrName,attrValue)
     return WrappedPickle.dumps((ret,None))
@@ -2117,7 +2137,7 @@ def pauseTask(req,jediTaskID):
     if 'SSL_CLIENT_S_DN' in req.subprocess_env:
         user = _getDN(req)
     # check role
-    prodRole = _isProdRoleATLAS(req)
+    prodRole = _hasProdRole(req)
     # check jediTaskID
     try:
         jediTaskID = long(jediTaskID)
@@ -2137,7 +2157,7 @@ def resumeTask(req,jediTaskID):
     if 'SSL_CLIENT_S_DN' in req.subprocess_env:
         user = _getDN(req)
     # check role
-    prodRole = _isProdRoleATLAS(req)
+    prodRole = _hasProdRole(req)
     # check jediTaskID
     try:
         jediTaskID = long(jediTaskID)
@@ -2157,7 +2177,7 @@ def avalancheTask(req,jediTaskID):
     if 'SSL_CLIENT_S_DN' in req.subprocess_env:
         user = _getDN(req)
     # check role
-    prodRole = _isProdRoleATLAS(req)
+    prodRole = _hasProdRole(req)
     # check jediTaskID
     try:
         jediTaskID = long(jediTaskID)
@@ -2215,7 +2235,7 @@ def changeTaskModTimePanda(req,jediTaskID,diffValue):
     if 'SSL_CLIENT_S_DN' in req.subprocess_env:
         user = _getDN(req)
     # check role
-    prodRole = _isProdRoleATLAS(req)
+    prodRole = _hasProdRole(req)
     # only prod managers can use this method
     if not prodRole:
         return WrappedPickle.dumps((False,"production or pilot role required"))
@@ -2284,7 +2304,7 @@ def reassignShare(req, jedi_task_ids_pickle, share, reassign_running):
     if 'SSL_CLIENT_S_DN' in req.subprocess_env:
         user = _getDN(req)
     # check role
-    prod_role = _isProdRoleATLAS(req)
+    prod_role = _hasProdRole(req)
     if not prod_role:
         return WrappedPickle.dumps((False,"production or pilot role required"))
 
@@ -2310,7 +2330,7 @@ def listTasksInShare(req, gshare, status):
     if 'SSL_CLIENT_S_DN' in req.subprocess_env:
         user = _getDN(req)
     # check role
-    prod_role = _isProdRoleATLAS(req)
+    prod_role = _hasProdRole(req)
     if not prod_role:
         return WrappedPickle.dumps((False,"production or pilot role required"))
 
@@ -2444,7 +2464,7 @@ def setNumSlotsForWP(req, pandaQueueName, numSlots, gshare=None, resourceType=No
     if not isSecure(req):
         return json.dumps((100, "SSL is required"))
     # check role
-    if not _isProdRoleATLAS(req):
+    if not _hasProdRole(req):
         return json.dumps((101, "production role is required in the certificate"))
     # convert
     try:
@@ -2461,7 +2481,7 @@ def enableJumboJobs(req, jediTaskID, nJumboJobs, nJumboPerSite=None):
     if not isSecure(req):
         return json.dumps((100, "SSL is required"))
     # check role
-    if not _isProdRoleATLAS(req):
+    if not _hasProdRole(req):
         return json.dumps((101, "production role is required in the certificate"))
     # convert
     try:
@@ -2514,7 +2534,7 @@ def sweepPQ(req, panda_queue, status_list, ce_list, submission_host_list):
     if not isSecure(req):
         return json.dumps((False,"SSL is required"))
     # check role
-    prod_role = _isProdRoleATLAS(req)
+    prod_role = _hasProdRole(req)
     if not prod_role:
         return json.dumps((False, "production or pilot role required"))
 
@@ -2535,9 +2555,12 @@ def decode_idds_enum(d):
 
 # relay iDDS command
 def relay_idds_command(req, command_name, args=None, kwargs=None, manager=None):
+    tmpLog = LogWrapper(_logger, 'relay_idds_command-{}'.format(datetime.datetime.utcnow().isoformat('/')))
     # check security
     if not isSecure(req):
-        return json.dumps((False, "SSL is required"))
+        tmpStr = "SSL is required"
+        tmpLog.error(tmpStr)
+        return json.dumps((False, tmpStr))
     try:
         if manager is not True:
             manager = False
@@ -2550,19 +2573,22 @@ def relay_idds_command(req, command_name, args=None, kwargs=None, manager=None):
         else:
             c = iDDS_Client(idds_host)
         if not hasattr(c, command_name):
-            return json.dumps((False, "{} is not a command of iDDS {}".format(
-                command_name, c.__class__.__name__)))
+            tmpStr = "{} is not a command of iDDS {}".format(command_name, c.__class__.__name__)
+            tmpLog.error(tmpStr)
+            return json.dumps((False, tmpStr))
         if args:
             try:
                 args = idds.common.utils.json_loads(args)
-            except Exception:
+            except Exception as e:
+                tmpLog.warning('failed to load args json with {}'.format(str(e)))
                 args = json.loads(args, object_hook=decode_idds_enum)
         else:
             args = []
         if kwargs:
             try:
                 kwargs = idds.common.utils.json_loads(kwargs)
-            except Exception:
+            except Exception as e:
+                tmpLog.warning('failed to load kwargs json with {}'.format(str(e)))
                 kwargs = json.loads(kwargs, object_hook=decode_idds_enum)
         else:
             kwargs = {}
@@ -2571,13 +2597,18 @@ def relay_idds_command(req, command_name, args=None, kwargs=None, manager=None):
             dn = req.subprocess_env.get('SSL_CLIENT_S_DN')
             if dn:
                 kwargs['username'] = clean_user_id(dn)
-        _logger.debug("relay_idds_command : class={} com={} host={} args={} kwargs={}".format(
-            c.__class__.__name__, command_name, idds_host, str(args), str(kwargs)))
+        tmpLog.debug("execute: class={} com={} host={} args={} kwargs={}".format(
+            c.__class__.__name__, command_name, idds_host, str(args)[:200], str(kwargs)[:200]))
         ret = getattr(c, command_name)(*args, **kwargs)
-        return json.dumps((True, ret))
+        tmpLog.debug("ret: {}".format(str(ret)[:200]))
+        try:
+            return json.dumps((True, ret))
+        except Exception:
+            return idds.common.utils.json_dumps((True, ret))
     except Exception as e:
-        _logger.error("relay_idds_command : %s %s" % (str(e), traceback.format_exc()))
-        return json.dumps((False, 'server failed with {}'.format(str(e))))
+        tmpStr = 'failed to execute command with {}'.format(str(e))
+        tmpLog.error("{} {}".format(tmpStr, traceback.format_exc()))
+        return json.dumps((False, tmpStr))
 
 
 # relay iDDS workflow command with ownership check
@@ -2612,7 +2643,7 @@ def execute_idds_workflow_command(req, command_name, kwargs=None):
             if not dn:
                 tmpMsg = 'SSL_CLIENT_S_DN is missing in HTTP request'
                 tmpLog.error(tmpMsg)
-                return json.dumps((False, ))
+                return json.dumps((False, tmpMsg))
             requester = clean_user_id(dn)
             # get request_id
             request_id = kwargs.get('request_id')
@@ -2646,8 +2677,33 @@ def send_command_to_job(req, panda_id, com):
     if not isSecure(req):
         return json.dumps((False,"SSL is required"))
     # check role
-    prod_role = _isProdRoleATLAS(req)
+    prod_role = _hasProdRole(req)
     if not prod_role:
         return json.dumps((False, "production or pilot role required"))
-
     return json.dumps(userIF.send_command_to_job(panda_id, com))
+
+
+# set user secret
+def set_user_secret(req, key=None, value=None):
+    tmpLog = LogWrapper(_logger, 'set_user_secret-{}'.format(datetime.datetime.utcnow().isoformat('/')))
+    # get owner
+    dn = req.subprocess_env.get('SSL_CLIENT_S_DN')
+    if not dn:
+        tmpMsg = 'SSL_CLIENT_S_DN is missing in HTTP request'
+        tmpLog.error(tmpMsg)
+        return json.dumps((False, tmpMsg))
+    owner = clean_user_id(dn)
+    return json.dumps(userIF.set_user_secret(owner, key, value))
+
+
+# get user secrets
+def get_user_secrets(req):
+    tmpLog = LogWrapper(_logger, 'get_user_secrets-{}'.format(datetime.datetime.utcnow().isoformat('/')))
+    # get owner
+    dn = req.subprocess_env.get('SSL_CLIENT_S_DN')
+    if not dn:
+        tmpMsg = 'SSL_CLIENT_S_DN is missing in HTTP request'
+        tmpLog.error(tmpMsg)
+        return json.dumps((False, tmpMsg))
+    owner = clean_user_id(dn)
+    return json.dumps(userIF.get_user_secrets(owner))
